@@ -2,7 +2,7 @@
 # Copyright (c) 2022 MBition GmbH
 
 from dataclasses import dataclass, field
-from typing import Optional, List
+from typing import Optional, List, Dict, Any
 from odxtools.utils import read_description_from_odx
 from .odxlink import OdxLinkRef, OdxLinkId, OdxDocFragment, OdxLinkDatabase
 
@@ -44,6 +44,12 @@ class AdditionalAudience:
     long_name: Optional[str] = None
     description: Optional[str] = None
 
+    def _build_odxlinks(self) -> Dict[OdxLinkId, Any]:
+        return { self.odx_id: self }
+
+    def _resolve_references(self, odxlinks: OdxLinkDatabase) -> None:
+        pass
+
 
 def read_audience_from_odx(et_element, doc_frags: List[OdxDocFragment]):
     enabled_audience_refs = [OdxLinkRef.from_et(ref, doc_frags)
