@@ -14,7 +14,8 @@ class TableStructParameter(Parameter):
                  byte_position=None,
                  bit_position=None,
                  semantic=None,
-                 description=None):
+                 description=None,
+                 **kwargs):
         super().__init__(
             short_name=short_name,
             long_name=long_name,
@@ -22,7 +23,8 @@ class TableStructParameter(Parameter):
             bit_position=bit_position,
             parameter_type="TABLE-STRUCT",
             semantic=semantic,
-            description=description
+            description=description,
+            **kwargs
         )
         if table_key_ref:
             self.table_key_ref = table_key_ref
